@@ -12,8 +12,8 @@
 
 import unittest
 
-import six
 import ddt
+import six
 
 from tempest.lib import exceptions
 
@@ -27,7 +27,7 @@ class CinderVolumeNegativeTests(base.ClientTestBase):
     @ddt.data(
         ('', (r'Size is a required parameter')),
         ('-1', (r'Invalid volume size provided for create request')),
-        ('0', (r'Invalid input received')),
+        ('0', (r"Volume size '0' must be an integer and greater than 0")),
         ('size', (r'invalid int value')),
         ('0.2', (r'invalid int value')),
         ('2 GB', (r'unrecognized arguments')),
@@ -108,7 +108,7 @@ class CinderVolumeTestsWithParameters(base.ClientTestBase):
         1) create volume with metadata
         2) check that metadata complies entered
         """
-        volume = self.object_create('volume',
-                                params='--metadata test_metadata=test_date 1')
-        self.assertEqual("{u'test_metadata': u'test_date'}",
+        volume = self.object_create(
+            'volume', params='--metadata test_metadata=test_date 1')
+        self.assertEqual(six.text_type({u'test_metadata': u'test_date'}),
                          volume['metadata'])
