@@ -14,6 +14,7 @@
 #    under the License.
 
 from __future__ import print_function
+import collections
 
 import os
 import pkg_resources
@@ -36,6 +37,15 @@ def arg(*args, **kwargs):
     return _decorator
 
 
+def exclusive_arg(group_name, *args, **kwargs):
+    """Decorator for CLI mutually exclusive args."""
+    def _decorator(func):
+        required = kwargs.pop('required', None)
+        add_exclusive_arg(func, group_name, required, *args, **kwargs)
+        return func
+    return _decorator
+
+
 def env(*vars, **kwargs):
     """
     returns the first environment variable set
@@ -62,6 +72,24 @@ def add_arg(f, *args, **kwargs):
         f.arguments.insert(0, (args, kwargs))
 
 
+def add_exclusive_arg(f, group_name, required, *args, **kwargs):
+    """Bind CLI mutally exclusive arguments to a shell.py `do_foo` function."""
+
+    if not hasattr(f, 'exclusive_args'):
+        f.exclusive_args = collections.defaultdict(list)
+        # Default required to False
+        f.exclusive_args['__required__'] = collections.defaultdict(bool)
+
+    # NOTE(sirp): avoid dups that can occur when the module is shared across
+    # tests.
+    if (args, kwargs) not in f.exclusive_args[group_name]:
+        # Because of the semantics of decorator composition if we just append
+        # to the options list positional options will appear to be backwards.
+        f.exclusive_args[group_name].insert(0, (args, kwargs))
+        if required is not None:
+            f.exclusive_args['__required__'][group_name] = required
+
+
 def unauthenticated(f):
     """
     Adds 'unauthenticated' attribute to decorated function.
@@ -159,7 +187,7 @@ def print_list(objs, fields, exclude_unavailable=False, formatters=None,
 
 def _encode(src):
     """remove extra 'u' in PY2."""
-    if six.PY2 and isinstance(src, unicode):
+    if six.PY2 and isinstance(src, six.text_type):
         return src.encode('utf-8')
     return src
 
